/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2002 Dave Smith & Julian Missig
 *  Copyright (c) 2002 Julian Missig
 */

#include "RosterView.hh"

#include "ContactInfo.hh"
#include "Environment.hh"
#include "JabberConnection.hh"
#include "GabberUtility.hh"
#include "Menus.hh"
#include "RosterModel.hh"

#include <gtkmm/messagedialog.h>
#include <gtkmm/stock.h>

namespace Gabber {

// -------------------------------------------------------------------
//
// GabberWin's Roster View
//
// -------------------------------------------------------------------

RosterView::RosterView(JabberConnection& jc, Gtk::TreeView* treeview, 
    Glib::RefPtr<RosterModel> rostermodel, Gtk::Window* parent) : 
    _jabber_connection(jc), _treeview(treeview), _rostermodel(rostermodel),
    _parent(parent), _on_jid(false)
     
{
     // Set the model
     _treeview->set_model(_rostermodel);

     _rselection = _treeview->get_selection();
     _rselection ->set_mode(Gtk::SELECTION_BROWSE);

     add_columns();
     _treeview->signal_cursor_changed().connect(
             SigC::slot(*this, &RosterView::on_signal_cursor_changed));

     // Connect to the row activation
     _treeview->signal_row_activated().connect(SigC::slot(*this,
                 &RosterView::on_row_activated));

     // Connect to a button press signal the gtkmm way (Doesn't work): 
     _treeview->signal_button_press_event().connect(
             SigC::slot(*this, &RosterView::on_button_press_event), false);

     _treeview->signal_event().connect(
             SigC::slot(*this, &RosterView::on_event));

    _user_menu = new Popups::User();
    Popups::User::getSingleton().getItem("get_info")->signal_activate().connect(SigC::slot(*this, &RosterView::on_view_info));
    Popups::User::getSingleton().getItem("delete_this_contact")->signal_activate().connect(SigC::slot(*this, &RosterView::on_delete_contact));

    _rostermodel->signal_row_inserted().connect(SigC::slot(*this,
        &RosterView::on_row_inserted));
}

RosterView::~RosterView()
{
    delete _user_menu;
}

void RosterView::add_columns()
{
     // Show Icon and Name
    {
        Gtk::TreeView::Column* pColumn = Gtk::manage( new
            Gtk::TreeView::Column("Name") );

        pColumn->pack_start(_rostermodel->columns.icon, false);
        pColumn->pack_start(_rostermodel->columns.name, true);

        std::vector<Gtk::CellRenderer*> rends = pColumn->get_cell_renderers();
        pColumn->clear_attributes(*rends[1]);
        pColumn->add_attribute(*rends[1], "markup", 1);
        pColumn->add_attribute(rends[0]->property_visible(),
                               _rostermodel->columns.use_icon);

        int cols_count = _treeview->append_column(*pColumn);
    }
}

bool RosterView::on_event(GdkEvent* e) { 
// FIXME: somehow we need to figure out the user's favorite view type here; 
    bool was_handled = false;// Single-click
    if(e->type == GDK_BUTTON_PRESS) {
        if (e->button.button == 3) {  // Check for right-click
        // FIXME: do stuff for agent JIDs here
        // Only popup the context menu for regular JIDs
        if (_on_jid)
            Popups::User::getSingleton().popup(e->button.button, e->button.time);
        }
    }
    return was_handled;
}

void RosterView::on_row_activated(const Gtk::TreeModel::Path& path, 
        Gtk::TreeViewColumn* column)
{
    Gtk::TreeModel::iterator iter = _rostermodel->get_iter(path);
    // It's a group, toggle it's state
    if (iter->get_value(_rostermodel->columns.is_group))
    {
        Glib::ustring name = iter->get_value(_rostermodel->columns.name);
        if (_treeview->row_expanded(path))
        {
            _treeview->collapse_row(path);
            //_open_groups.erase(name);
        }
        else
        {
            _treeview->expand_row(path, false);
            //_open_groups.insert(name);
        }
        return;
    }

    // Do what the menu wants us to do!
    Popups::User::getSingleton().items().front().activate();
}

bool RosterView::on_button_press_event(GdkEventButton* b) { 
    return false;
}

// This will be called when a row in the roster view is highlighted
void RosterView::on_signal_cursor_changed(){
    Gtk::TreeRow row    = *(_treeview->get_selection()->get_selected());
    Glib::ustring full_jid;
    try 
    {
        Glib::ustring roster_jid;
        row.get_value(0, roster_jid);
        full_jid =
        _jabber_connection.getSession().presenceDB().find(roster_jid)->getFrom();
    } 
    catch (jabberoo::PresenceDB::XCP_InvalidJID& xcp) 
    {
        row.get_value(0, full_jid);
    }

    if (!full_jid.empty())
        _on_jid = true;

    Popups::User::getSingleton().updateSelectedJID(full_jid);
}

void RosterView::on_view_info()
{
     const Glib::ustring& cur_jid = Popups::User::getSingleton().getSelectedJID();
     ContactInfoDlg::display(_jabber_connection, *_parent, cur_jid);
}

void RosterView::on_delete_contact()
{
    jabberoo::Session& session = _jabber_connection.getSession();
    jabberoo::Roster& roster = _jabber_connection.getSession().roster();
    const Glib::ustring& cur_jid = Popups::User::getSingleton().getSelectedJID();
    jabberoo::Roster::Item item = roster[cur_jid];
    jabberoo::Roster::Subscription s10n = item.getSubsType();

    if (s10n == jabberoo::Roster::rsFrom || s10n == jabberoo::Roster::rsBoth)
    {
        session << jabberoo::Presence(cur_jid, 
            jabberoo::Presence::ptUnsubscribed);
    }

    // Lookup nickname
    Glib::ustring nickname;
    // XXX: Maybe this should be in Popups::User::getSingleton() as well?
    // attempt to look it up
    try {
	 nickname = _jabber_connection.getSession().roster()[jabberoo::JID::getUserHost(cur_jid)].getNickname();
    } catch (jabberoo::Roster::XCP_InvalidJID& e) {
	 // the default nickname is the username
	 nickname = jabberoo::JID::getUser(cur_jid);
    }

    // Confirmation Dialog
    Gtk::MessageDialog dlg(*_parent, 
			   Util::substitute(_("Are you sure you want to remove %s (%s) from your Contact List?"), nickname, cur_jid),
			   Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE, true);
    dlg.set_title(Util::substitute(_("Remove %s from Contact List"), nickname));
    dlg.add_button(_("_Don't Remove"), Gtk::RESPONSE_NO);
    dlg.add_button(Gtk::Stock::REMOVE, Gtk::RESPONSE_YES);

    int ret = dlg.run();
    dlg.hide();
    if (ret == Gtk::RESPONSE_YES)
	 roster.deleteUser(cur_jid);
}

void RosterView::on_row_inserted(const Gtk::TreeModel::Path& path, 
                                 const Gtk::TreeModel::iterator& iter)
{
    #if 0
    // This is lame, we can't actually see if it is a group and then expand it
    // because expansion only takes effect if the row has children.
    if ( !iter->get_value(_rostermodel->columns.is_group) )
    {
        Gtk::TreeModel::iterator parent_it = iter->parent();
        Gtk::TreeModel::Path parent_path = _rostermodel->get_path(parent_it);
        if (!_treeview->row_expanded(parent_path) &&
            (_open_groups.find(parent_it->get_value(
                _rostermodel->columns.name)) != _open_groups.end()))
        {
            _treeview->expand_row(parent_path, true);
        }
    }
    #endif
}

} // namespace Gabber
